".makeRotations" <- function(theta) {
    det12 = sin(theta[2])*cos(theta[1])-cos(theta[2])*sin(theta[1])
    det34 = sin(theta[4])*cos(theta[3])-cos(theta[4])*sin(theta[3])
    U12 = matrix(
      c( sin(theta[2]),-cos(theta[2]),
        -sin(theta[1]), cos(theta[1])),ncol=2)/det12
    U34 = matrix(
      c( sin(theta[4]),-cos(theta[4]),
        -sin(theta[3]), cos(theta[3])),ncol=2)/det34
    list(a=U12,b=U34)
}

setGeneric("DeCorrelateChannels",
           function(int,...)
           standardGeneric("DeCorrelateChannels"),
           signature = "int")
setMethod("DeCorrelateChannels",
          signature = signature(int = "SolexaIntensity"),
          function(int,cycles=seq(1,dim(int)[3],by=1),
                   theta=matrix(rep(c(0.8806742,1.3727418,0.883865,1.545728),length(cycles)),ncol=4,byrow=TRUE))
          {
              int.rtn = int
              for (c in seq(along=cycles)) {
                  U = .makeRotations(theta[c,])
                  cy = cycles[c]
                  int.rtn@intensity@.Data[,c('A','C'),cy] = int@intensity@.Data[,c('A','C'),cy] %*% U$a
                  int.rtn@intensity@.Data[,c('G','T'),cy] = int@intensity@.Data[,c('G','T'),cy] %*% U$b
              }
              return(int.rtn)
          })
setMethod("DeCorrelateChannels",
          signature = signature(int = "array"),
          function(int,cycles=seq(1,dim(int)[3],by=1),
                   theta=matrix(rep(c(0.8806742,1.3727418,0.883865,1.545728),length(cycles)),ncol=4,byrow=TRUE))
          {
              int.rtn = int
              for (c in seq(along=cycles)) {
                  U = .makeRotations(theta[c,])
                  cy = cycles[c]
                  int.rtn[,c('A','C'),cy] = int[,c('A','C'),cy] %*% U$a
                  int.rtn[,c('G','T'),cy] = int[,c('G','T'),cy] %*% U$b
              }
              return(int.rtn)
          })

setGeneric("OptimizeAngle",
           function(int,...)
           standardGeneric("OptimizeAngle"),
           signature = "int")
setMethod("OptimizeAngle",
          signature = signature(int = "SolexaIntensity"),
          function(int,cycles=seq(1,dim(int)[3],by=1),...)
          {
              int0 = as(intensity(int),"array")
              angle = matrix(0,ncol=4,nrow=length(cycles))
              for (c in seq(along=cycles)) {
                  cy = cycles[c]
                  angle[c,] = optim(par=.theta0,
                         fn=function(theta) {
                             intBC = DeCorrelateChannels(int0,cycles=cy,theta=t(as.matrix(theta)))
                             r2 = cor(intBC[,'A',cy],intBC[,'C',cy])^2
                             r2 = r2+cor(intBC[,'G',cy],intBC[,'T',cy])^2
                             if (is.na(r2)) r2=2
                             r2
                         },method="L-BFGS-B",
                         lower=pi*c(0,.25,0,.25),upper=pi*c(.25,.5,.25,.5),...)$par
              }
              return(angle)
          })

setGeneric("DeCorrelateCycles",
           function(int,...)
           standardGeneric("DeCorrelateCycles"),
           signature = "int")
setMethod("DeCorrelateCycles",
          signature = signature(int = "SolexaIntensity"),
          function(int,ncycles=dim(int)[3],rate=1.8e-2)
          {
              mixture = matrix(0,ncol=ncycles,nrow=ncycles)
              for (i in 1:ncycles) {
                  mixture[i,i] = (1-rate)^i
                  for (j in 1:(i-1)) {
                      mixture[i,j] = choose(i-1,i-j)*rate^(i-j)*(1-rate)^j
                  }
              }
              intDC = int
              for (n in 1:4) {
                  intDC@intensity@.Data[,n,1:ncycles] =
                    t(forwardsolve(mixture,t(intDC@intensity@.Data[,n,1:ncycles])))
              }
              return(intDC)
          })
setMethod("DeCorrelateCycles",
          signature = signature(int = "array"),
          function(int,ncycles=dim(int)[3],rate=1.8e-2)
          {
              mixture = matrix(0,ncol=ncycles,nrow=ncycles)
              for (i in 1:ncycles) {
                  mixture[i,i] = (1-rate)^i
                  for (j in 1:(i-1)) {
                      mixture[i,j] = choose(i-1,i-j)*rate^(i-j)*(1-rate)^j
                  }
              }
              intDC = int
              for (n in 1:4) {
                  intDC[,n,1:ncycles] = t(forwardsolve(mixture,t(intDC[,n,1:ncycles])))
              }
              return(intDC)
          })

setGeneric("OptimizeRate",
           function(int,...)
           standardGeneric("OptimizeRate"),
           signature = "int")
setMethod("OptimizeRate",
          signature = signature(int = "SolexaIntensity"),
          function(int,ncycles=dim(int)[3],...)
          {
              int0 = as(intensity(int),"array")
              rate = optim(par=1.8e-2,
                fn=function(rate) {
                    intDC = DeCorrelateCycles(int=int0,ncycles=ncycles,rate=rate)
                    correl = 0
                    for (k in 1:(ncycles-1))
                      for (j in 1:4)
                        correl = correl+cor(intDC[,j,k],intDC[,j,k+1])^2
                    correl
                },method="L-BFGS-B",lower=0,upper=.2,...)$par
              return(rate)
          })


".InitInt" <- function(int) {
    class = matrix(0,ncol=4,nrow=nrow(int))
    for (n in 1:nrow(class)) {
        ### which.max doesn't find multiple answers
        nc = which(int[n,] == max(int[n,]))
        class[n,nc] = 1/length(nc)
    }
    class
}

".InitSeq" <- function(seq) {
    class = matrix(0,ncol=4,nrow=length(seq))
    map = strsplit(IUPAC_CODE_MAP[seq],split='')
    for (n in 1:nrow(class)) {
        nc = which(c('A','C','G','T') %in% map[[n]])
        class[n,nc] = 1/length(nc)
    }
    class
}

setGeneric("TileNormalize",
           function(run,...)
           standardGeneric("TileNormalize"),
           signature = "run")
setMethod("TileNormalize",
          signature = signature(run = "RolexaRun"),
          function(run=Rolexa.env,int,cycles=seq(1,dim(int)[3],by=1)) {
              int.corr = int
              coord = cbind(readInfo(int)$tile,readInfo(int)$x,readInfo(int)$y)
              tiles = unique(coord[,1])
              int = as(intensity(int),"array")
              glob.median = apply(int,3,median)
              for (t in tiles) {
                  I = which(coord[,1] == t)
                  int2 = int[I,,]
                  if (length(I) < 100) {
                      if (run@verbose>0) {
                          print("To few coordinates, skipping normalization")
                      }
                      next
                  }
                  x = coord[I,2]
                  y = coord[I,3]
                  for (s in cycles) {
                      z=int2[,,s]
                      int.lo = loess(z~x+y, span=.2,
                        control=loess.control(surface="interpolate",
                          statistics="approximate",
                          trace.hat="approximate"))
                      int2[,,s] = z-predict(int.lo)+glob.median[s]
                  }
                  Ina = which(is.na(int2))
                  int2[Ina] = int[I[Ina]]
                  int.corr@intensity@.Data[I,,] = int2
              }
              int.corr
          })

setGeneric("SeqScore",
           function(run,...) standardGeneric("SeqScore"),
           signature = "run")
setMethod("SeqScore",
          signature = signature(run = "RolexaRun"),
          function(run=Rolexa.env,int,seqInit,colonies,cycles,plot=FALSE)
          {
              if (!require(mclust)) stop("Need mclust package")
              ht = run@HThresholds
              ncol = min(run@SequencingLength,dim(int)[3])
              wseq = FALSE
              icoord = cbind(readInfo(int)$lane,
                readInfo(int)$tile,
                readInfo(int)$x,
                readInfo(int)$y)
              if (missing(cycles)) cycles = 1:dim(int)[3]
              if (ncol<length(cycles)) cycles = cycles[1:ncol]
              if (missing(colonies) || length(colonies)==0) colonies = 1:dim(int)[1]
              int = int[colonies]
              icoord = icoord[colonies,]
              if ((!missing(seqInit)) && (length(seqInit))>0) {
                  wseq = (length(seqInit)>0)
                  siseq = seqInit@sread[colonies]
                  if (wseq) {
                      ncol = min(ncol,median(nchar(siseq)))
                      IO = .MatchCoordinates(run@idsep,seqInit,int)
                      if (nrow(IO)<100) {
                          seqInit = new(class(seqInit))
                          siseq = c()
                          wseq = FALSE
                      }
                  }
              }
              if (plot) {
                  cols = run@colors[c(2:5,16)]
              } else {
                  seq = matrix('',nrow=length(cycles),ncol=length(colonies))
                  entropy = matrix(0,nrow=length(cycles),ncol=length(colonies))
              }
              cmin = 0
              if (run@PET) cmin = (ncol-run@Barcode)/2
              cmax = cmin+run@Barcode+1
              if (run@fit) {
                  c = 1
                  if (run@Barcode) c = cmax
                  data = int@intensity@.Data[,,c]
                  if (wseq) {
                      mcl = meEEV(data=data[IO[,1],],z=.InitSeq(substr(siseq[IO[,2]],c,c)))
                  } else {
                      mcl = meEEV(data=data,z=.InitInt(data))
                  }
              }
              for (c in cycles) {
                  data = int@intensity@.Data[,,c]
                  
#                  if (run@Barcode && c > cmin && c < cmax) {
#                      c = cmax
#                      if (c == cmin+1) {
#                          data1 = int@intensity@.Data[,,cmax]
#                          if (wseq) {
#                              seq1 = substr(siseq[IO[2]],cmax,cmax)
#                              mcl = mstepEEV(data=data1[OIi,],z=.InitSeq(seq1))
#                          } else {
#                              mcl = mstepEEV(data=data1,z=.InitInt(data1))
#                          }
#                      }
#                  }

                  
                  if (run@fit) {
                      mcl = emEEV(data=data,parameters=mcl$parameters)
                  } else {
                      if (wseq) {
                          mcl = mstepEEV(data=data[IO[,1],],z=.InitSeq(substr(siseq[IO[,2]],c,c)))
                      } else {
                          mcl = mstepEEV(data=data,z=.InitInt(data))
                      }
                      mcl = estepEEV(data=data,parameters=mcl$parameters)
                  }
                  z = mcl$z
                  if (plot) {
                      class = map(mcl$z)
                      coordProj(data=data,dimens=c(1,2),
                                parameters=mcl$parameters,classification=class,
                                colors=cols,symbols='.')
                      title(main=c,xlab="A",ylab="C")
                      coordProj(data=data,dimens=c(3,4),
                                parameters=mcl$parameters,classification=class,
                                colors=cols,symbols='.')
                      title(main='',xlab="G",ylab="T")
                  } else {
                      entropy[c,] = -apply(z*log2(z),1,sum,na.rm=TRUE)
                      for (n in 1:ncol(entropy)) {
                          h = entropy[c,n]
                          po = DNA_ALPHABET[order(z[n,])]
                          if      (h < ht[1]) seq[c,n] = po[4]
                          else if (h < ht[2]) seq[c,n] = paste(sort(po[3:4]),collapse='')
                          else if (h < ht[3]) seq[c,n] = paste(sort(po[-1]),collapse='')
                          else                seq[c,n] = 'ACGT'
                      }
                  }
              }
              if (plot) return(1)
              for (a in names(IUPAC_CODE_MAP)) seq[which(seq==IUPAC_CODE_MAP[a])] = a
              seq.out = apply(seq,2,paste,collapse='')
              name = ''
              if (wseq) {
                  name = strsplit(as.character(id(seqInit)[1]),split=':')[[1]][1]
              }
              return(list(id=cbind(name,icoord),
                          sread=DNAStringSet(seq.out),
                          entropy=t(entropy)))
          })

setGeneric("FilterResults",
           function(run,...)
           standardGeneric("FilterResults"),
           signature = "run")
setMethod("FilterResults",
          signature = signature(run = "RolexaRun"),
          function(run=Rolexa.env,results) {
              len = run@MinimumTagLength
              it = run@IThresholds
              nt = DNA_ALPHABET
              ht1 = run@HThresholds[1]
              bc = run@Barcode
              Iselect = c()
              nrow = nrow(results$entropy)
              ncycles = ncol(results$entropy)
              if (ncycles<len) {
                  print(paste("Too few cycles for requesting minimum tag length =",len,
                              ", setting it to",ncycles))
                  len=ncycles
              }
              str = as.matrix(results$sread)
#              pre.entropy  = results$entropy
              coords = results$id
              post.entropy = matrix(log2(nchar(IUPAC_CODE_MAP[str])),nrow=nrow,ncol=ncycles)
              if (run@PET) {
                  n1 = (ncycles-bc)/2
                  ncycles = n1+bc
                  read1 = c((n1+1):ncycles,1:n1)
                  read2 = n1+1:ncycles
#                  pre.entropy = rbind(pre.entropy[,read1],pre.entropy[,read2])
                  post.entropy = rbind(post.entropy[,read1],post.entropy[,read2])
                  str = rbind(str[,read1],str[,read2])
              }
              if (len > length(it)) {
                  I = 1:nrow
                  if (bc) {
                      results$sread = DNAStringSet(apply(str[I,-(1:bc)],1,paste,collapse=''))
                      results$barcode = DNAStringSet(apply(str[I,1:bc],1,paste,collapse=''))
                  }
                  if (run@PET) {
                      results$sread2 = DNAStringSet(read2 = apply(str[nrow+I,],1,paste,collapse=''))
                  }
                  return(results)
              }
              for (c in 1:nrow(post.entropy)) {
                  score = sum(post.entropy[c,bc+1:len])
                  min.score = score
                  min.ind = 1+bc
                  if (ncycles > len+bc) {
                      for (i in (bc+1):(ncycles-len)) {
                          score = score-post.entropy[c,i]+post.entropy[c,i+len]
                          if (score < min.score) {
                              min.score = score
                              min.ind = i+1
                          }
                      }
                  }
                  score = min.score
                  i = min.ind
                  j = i+len-1
                  if (score > it[len]) {
                      if (run@PET) {
                          str[c,] = ''
                          results$entropy[c,] = -1
                      }
                      next
                  }
                  Iselect = c(Iselect,c)
                  len2 = len
                  while (len2 < ncycles) {
                      sc0 = -1
                      if (i-1>bc)    sc0 = it[len2+1]-score-post.entropy[c,i-1]
                      sc1 = -1
                      if (j<ncycles) sc1 = it[len2+1]-score-post.entropy[c,j+1]
                      n = which.max(c(sc0,sc1,0))
                      if (n == 1) {
                          i=i-1
                          score = score+post.entropy[c,i]
                      } else if (n == 2) {
                          j=j+1
                          score = score+post.entropy[c,j]
                      } else {
                          if (i>bc+1)
                            i = max(which(c(2,post.entropy[c,(bc+1):(i-1)])>=ht1))
                          if (j<ncycles) 
                            j = j-1+min(which(c(post.entropy[c,-(1:j)],2)>=ht1))
                          score = sum(post.entropy[c,i:j])
                          break
                      }
                      len2 = j-i+1
                  }
                  str.keep = i:j
                  if (bc > 0 && ((!run@PET) || (c <= nrow)))
                    str.keep = c(1:bc,str.keep)
                  str[c,-str.keep] = ''
#                  pre.entropy[c,1] = sum(pre.entropy[c,i:j])
                  results$entropy[c,-(i:j)] = -1
              }
              if (run@PET) {
                  I2 = which(Iselect > nrow(results))
                  Iselect[I2] = Iselect[I2]-nrow
                  Iselect = unique(sort(Iselect))
              }
              if (bc > 0) {
                  read1 = apply(str[Iselect,-(1:bc)],1,paste,collapse='')
                  barcode = apply(str[Iselect,1:bc],1,paste,collapse='')
              } else {
                  read1 = apply(str[Iselect,],1,paste,collapse='')
              }
              if (run@PET) read2 = apply(str[nrow+Iselect,],1,paste,collapse='')
              results$id = coords[Iselect,]
              results$sread = DNAStringSet(read1)
              results$entropy = results$entropy[Iselect,]
              if (bc > 0) results$barcode = DNAStringSet(barcode)
              ##### whole entropy list!
#              results$entropy = cbind(pre.entropy[Iselect,1],post.entropy[Iselect,1])
              if (run@PET) {
#                  results$entropy = cbind(
#                    results$entropy,
#                    pre.entropy[nrow+Iselect,1],
#                    post.entropy[nrow+Iselect,1])
                  results$sread2 = DNAStringSet(read2)
              }
              return(results)
          })
