"QualityBoxPlots" <- function(run=Rolexa.env, seq, cycles, par=list(las=2)) {
    if (length(par)) op = par(par)
    scores = as(quality(seq),"matrix")
    scoord = .IdToCoordinates(run@idsep,seq)
    lanes = paste(unique(scoord$coord[,1]),collapse=":")
    tiles = paste(unique(scoord$coord[,2]),collapse=":")
    ncycles = ncol(scores)
    if (missing(cycles)) cycles = 1:ncycles
    main = paste(scoord$name,", Lane ",lanes,", Tile ",tiles,sep='')
    boxplot(as.data.frame(scores[,cycles]),names=cycles,
            xlab="Cycles",ylab="Quality",main=main,pch=' ')
    if (length(par)) par(op)
}


setGeneric("BatchAnalysis",
           function(run,...) standardGeneric("BatchAnalysis"),
           signature = "run")
setMethod("BatchAnalysis",
          signature = signature(run = "RolexaRun"),
          function(run=Rolexa.env, seq, scores, what=c("length","information","base","ratio","iupac"), main='')
          {
              if (length(what) > 1) what = what[1]
              w = charmatch( what, .batch.optlist, nomatch=1 )
              if (!w) stop("Ambiguous 'what' parameter")
              what = .batch.optlist[w]
              switch (EXPR = what, 
                      length = {
                          br1 = run@MinimumTagLength:run@SequencingLength
                          hist(nchar(seq),breaks=sort(c(br1-.25,br1+.25)),
                               col='darkred',border='darkred',xlab='Tag Length',main=main)
                      }, information = {
                          xlab = 'Information content per base'
                          plot(density((2*nchar(seq)-scores[,1])/run@SequencingLength),
                               col='darkgreen',xlab=xlab,main=main)
                      }, base = , ratio = , iupac = {
                          nt = DNA_ALPHABET
                          ncol = run@SequencingLength
                          nrow = length(seq)
                          base.count = matrix(0,ncol=ncol,nrow=8)
                          for (n in 1:ncol) {
                              for (k in 1:4) base.count[k,n] = sum(substr(seq,n,n) == nt[k])
                              base.count[5,n] = sum(substr(seq,n,n) %in% nt[5:10])
                              base.count[6,n] = sum(substr(seq,n,n) %in% nt[11:14])
                              base.count[7,n] = sum(substr(seq,n,n) %in% nt[-(1:14)])
                              base.count[8,n] = nrow-sum(base.count[1:8,n])
                          }
                          xlab = 'Cycles'
                          base.sum = apply(base.count[1:4,],2,sum)
                          if (what == "base") {
                              plot(1:ncol,base.count[1,]/base.sum,
                                   pch=19,col=2,xlim=c(1,ncol),ylim=c(0,1),
                                   xlab=xlab,ylab='Ratio',main=main)
                              points(1:ncol,base.count[2,]/base.sum,pch=19,col=3)
                              points(1:ncol,base.count[3,]/base.sum,pch=19,col=4)
                              points(1:ncol,base.count[4,]/base.sum,pch=19,col=5)
                              abline(h=.25)
                              legend(x=ncol,y=1,legend=nt[1:4],col=run@colors[2:5],
                                     xjust=1,pch=19,cex=par()$cex)
                          } else if (what == "ratio") {
                              plot(1:ncol,base.count[1,]/base.count[4,],
                                   pch=19,xlim=c(1,ncol),ylim=c(2/3,1.5),log='y',
                                   col=run@colors[2],
                                   xlab=xlab,ylab='Ratio',main=main)
                              points(1:ncol,base.count[2,]/base.count[3,],pch=19,col=run@colors[3])
                              abline(h=1)
                              legend(x=ncol,y=2,legend=c("A/T","C/G"),col=run@colors[2:3],
                                     xjust=1,pch=19,cex=par()$cex)
                          } else {
                              legend = c("ACGT")
                              col = run@colors[2]
                              plot(1:ncol,base.sum/nrow,
                                   pch=19,col=col,xlim=c(1,ncol),ylim=c(0,1),
                                   xlab=xlab,ylab='Ratio',main=main)
                              if (any(base.count[5,]>0)) {
                                  legend = c(legend,"MRWSY")                    
                                  col = c(run@colors[3],col)
                                  points(1:ncol,base.count[5,]/nrow,pch=19,col=col[1])
                              }
                              if (any(base.count[6,]>0)) {
                                  legend = c(legend,"KVHD")
                                  col = c(run@colors[4],col)
                                  points(1:ncol,base.count[6,]/nrow,pch=19,col=col[1])
                              }
                              if (any(base.count[7,]>0)) {
                                  legend = c(legend,"N")
                                  col = c(run@colors[5],col)
                                  points(1:ncol,base.count[7,]/nrow,pch=19,col=col[1])
                              }
                              if (any(base.count[8,]>0)) {
                                  legend = c(legend,"-")
                                  col = c(run@colors[6],col)
                                  points(1:ncol,base.count[8,]/nrow,pch=19,col=col[1])
                              }
                              legend(x=1,y=.75,legend=legend,col=rev(col),
                                     pch=19,bg='white',cex=par()$cex)
                          }
                      })
          })

setGeneric("CombinedPlot",
           function(run,...) standardGeneric("CombinedPlot"),
           signature = c("run"))
setMethod("CombinedPlot",
          signature = signature(run = "RolexaRun"),
          function( run=Rolexa.env, int, seq, scores, colonies = 1:4, par = list() )
          {
              IO = .MatchCoordinates(run@idsep,seq,int)
              coord = cbind(readInfo(int)$tile[IO[colonies,1]],
                readInfo(int)$x[IO[colonies,1]],
                readInfo(int)$y[IO[colonies,1]])
              ncycles = min(dim(int)[3],ncol(score),max(nchar(sread(seq))))
              int = matrix(int@intensity@.Data[IO[colonies,1],,1:ncycles],nrow=length(colonies))
              
              seq = sread(seq)[IO[colonies,2]]
              score = matrix(scores[IO[colonies,1],1:ncycles],nrow=nrow(int))
              nt = DNA_ALPHABET
              if (length(par)) op = par(par)
              xpos = sort(rep(1:ncycles,4))+rep(c(-0.3,-0.1,0.1,0.3),ncycles)
              seqspl = as.matrix(seq)
              for (n in 1:nrow(int)) {
                  main = paste(coord[n,],collapse=':')
                  intn = int[n,]
                  plot(xpos,intn,t='h',col=run@colors[1:4],
                       xlim=c(1,ncycles), ylim=c(-2500,max(intn)),
                       xlab='',ylab='',main=main)
                  for (i in 1:ncycles) {
                      text(i,0,seqspl[n,i],pos=1,cex=par()$cex,
                           col=run@colors[which(nt == seqspl[n,i])])
                  }
                  range = 1500/max(score[n,])
                  lines(1:ncycles,score[n,]*range-2000)
              }
              if (length(par)) par(op)
          })
  

setGeneric("ChannelHistogram",
           function(int,...) standardGeneric("ChannelHistogram"),
           signature = "int")
setMethod("ChannelHistogram",
          signature = signature(int = "SolexaIntensity"),
          function( int, cycles = c(1,18,36), threemodes = FALSE, par = list() )
          {
              if (all(par()$mfrow == c(1,1))) {
                  if (length(cycles) < 7) par(mfrow = c(length(cycles),4))
                  else                    par(mfrow = c(6,4),ask = TRUE)
              }
              G = 2
              if (threemodes) G=3
              if (length(par)) op = par(par)
              for (i in cycles) {
                  for (n in c('A','C','G','T')) {
                      data = int@intensity@.Data[,n,i]
                      hist = Mclust(data,G=G,model="V")
                      mu = hist$par$mean
                      sigma = sqrt(hist$par$variance$sigmasq)
                      mclust1Dplot(data,parameters=hist$parameters,z=hist$z,
                                   what="density",xlab="Intensity",col='red',new=TRUE)
                      hist(data,freq=FALSE,breaks=100,add=TRUE)
                      title(main=n)
                      abline(v=mu[G],col='blue')
                      abline(v=mu[G]+c(-1,1)*sigma[G],col='blue',lty=2)
                      if (threemodes) {
                          abline(v=mu[2],col='cyan')
                          abline(v=mu[2]+c(-1,1)*sigma[2],col='cyan',lty=2)
                      }
                  }
              }
              if (length(par)) par(op)
          })


setGeneric("TileImage",
           function(int,...)
           standardGeneric("TileImage"),
           signature = "int")
setMethod("TileImage",
          signature = signature(int = "SolexaIntensity"),
          function(int,cycle,tile,channel=c('A','C','G','T'),ncell=30) {
              char = charmatch( channel, c('A','C','G','T') )
              if (!char) stop("Choose channel 'A','C','G' or 'T'")
              Itile = which(readInfo(int)$tile == tile)
              x = readInfo(int)$x[Itile]
              y = readInfo(int)$y[Itile]
              col = heat.colors(32)
              xf = cut(x=x, breaks=ncell)
              yf = cut(x=y, breaks=ncell)
              x = sapply(split(x,xf),mean)
              y = sapply(split(y,yf),mean)
              z = matrix(sapply(split(int@intensity@.Data[Itile,channel,cycle],list(xf,yf)),mean),
                nrow=ncell,byrow=TRUE)
              main=paste('Tile ',tile,', cycle ',cycle,sep='',', channel ',channel)
              image(x,y,z,main=main,col=col)
          })

setGeneric("PlotCycles",
           function(run,...) standardGeneric("PlotCycles"),
           signature = "run")
setMethod("PlotCycles",
          signature = signature(run = "RolexaRun"),
          function(run=Rolexa.env, int, seq, cycles=c(1,11,21,31), par=list())
          {
              if (par()$ask || all(par()$mfrow == c(1,1))) {
                  if (length(cycles) < 7) par(mfcol = c(2,length(cycles)))
                  else                    par(mfcol = c(2,6),ask = TRUE)
              }
              if (length(par)) op = par(par)
              if (!missing(seq) && length(seq)) {
                  SeqScore(run=run,int=int,cycles=cycles,seqInit=seq,plot=TRUE)
              } else {
                  SeqScore(run=run,int=int,cycles=cycles,plot=TRUE)
              }
              if (length(par)) par(op)
          })

