\name{BatchAnalysis} 

\alias{PlotCycles}
\alias{PlotCycles,RolexaRun-method}
\alias{BatchAnalysis}
\alias{BatchAnalysis,RolexaRun-method}
\alias{QualityBoxPlots}
\title{Batch Analysis}
\description{Generate summary plots of the results of a base calling batch} 
\usage{
\S4method{PlotCycles}{RolexaRun}(run=Rolexa.env, int, seq,
cycles=c(1,11,21,31), par=list())
PlotCycles(run,...)
\S4method{BatchAnalysis}{RolexaRun}(run=Rolexa.env, seq, scores, what=c("length","information","base","ratio","iupac"), main='')
BatchAnalysis(run,...)
QualityBoxPlots(run=Rolexa.env, seq, cycles, par=list(las=2))
}
\arguments{
  \item{run}{a \code{RolexaRun} object defining the run parameters}
  \item{int}{a \code{\link[ShortRead:SolexaIntensity-class]{SolexaIntensity}}
    object}
  \item{seq}{a \code{\link[Biostrings:XStringSet-class]{DNAStringSet}} object}
  \item{scores}{a matrix of base quality scores (one column per base,
    one row per sequence)}
  \item{what}{select one the plot types}
  \item{main}{a title for the plot}
  \item{cycles}{the cycles to plot}
  \item{par}{parameters for the plotting functions}
  \item{\dots}{additional arguments, ignored}
}
\details{
  Four types of diagnostic plots can be selected with the \code{what}
  argument of \code{BatchAnalysis}:
  \itemize{
    \item{\code{length}}{shows the
      histogram of tag lengths,}
    \item{\code{information}}{the distribution of information content per
      sequenced base, namely \code{((2*length(tag)-total_entropy(tag))/nb_cycles)},}
    \item{\code{base}}{the base composition of the sequences,}
    \item{\code{ratio}}{the ratio of
      complementary bases,}
    \item{\code{iupac}}{the proportion of the different classes of
      ambiguous bases along the sequences.}
  }
  \code{QualityBoxPlots} makes boxplots of quality scores along the sequences.
  \code{PlotCycles} will execute
  \code{\link{SeqScore}} with \code{plot=TRUE}.
}
\references{Probabilistic base calling of Solexa sequencing data,
  BMC Bioinformatics 2008, 9:431}
\author{Jacques Rougemont, Arnaud Amzallag, Christian Iseli, Laurent Farinelli, Ioannis Xenarios, Felix Naef}
\seealso{\code{\link{SaveResults}} to save the results produced by
  \code{\link{SeqScore}} or \code{\link{FilterResults}}.}
\examples{
path = SolexaPath(system.file("extdata", package="ShortRead"))
rolenv = SetModel(idsep="_")
int = readIntensities(path,pattern="s_1_0001",withVariability=FALSE)
seq = CombineReads(run=rolenv,path=path,pattern="s_1_0001_seq*")
results = SeqScore(run=rolenv,int=int,seqInit=seq,cycles=1:36)
PlotCycles(run=rolenv,int=int,seq=seq,cycles=1:4)
par(ask=TRUE)
BatchAnalysis(rolenv,sread(seq),matrix(),what="iupac")
BatchAnalysis(rolenv,sread(seq),results$entropy,what="information")
results = FilterResults(run=rolenv,results=results)
BatchAnalysis(rolenv,sread(seq),results,what="length")
seq = readFastq(path)
par(mar=c(4,4,1,1),cex=1.5,lwd=2)
QualityBoxPlots(rolenv,seq,cycles=10:36)
}
\keyword{utilities}
\keyword{dplot}
\keyword{manip}
